import { GetRequestBuilder } from './GetRequestBuilder';
import { itemsToObject } from 'shared-base';
import { NodeType } from 'redux-store-generator';
import { ResponseBuilder } from '../_base/ResponseBuilder';
export class RestAdapter {
    constructor(configuration) {
        this.GET = (request, response) => {
            const { argsPath, argsParams: inParams = {} } = request;
            const queryParams = { ...inParams };
            if (inParams.deep &&
                request.argsNodeType === NodeType.GROUPED_LIST_NODE) {
                const itemsKey = request.argsNodeName + 'Items';
                queryParams['_embed'] = itemsKey;
                delete queryParams['deep'];
                response.withTransformer((item) => {
                    const newItem = { ...item };
                    newItem['items'] = newItem[itemsKey];
                    delete newItem[itemsKey];
                    return newItem;
                });
            }
            const getRequest = new GetRequestBuilder()
                .withGetParams(queryParams)
                .withPath(argsPath)
                .build();
            const { path } = getRequest;
            return this.instance.get(path);
        };
        this.POST = (request, _response) => {
            const { argsPath, argsParams: data } = request;
            return this.instance.post(argsPath, data);
        };
        this.PATCH = (request, _response) => {
            const { argsPath, argsParams: data } = request;
            return this.instance.patch(argsPath, data);
        };
        this.PUT = (request, _response) => {
            const { argsPath, argsParams: data } = request;
            return this.instance.put(argsPath, data);
        };
        this.DELETE = (request, _response) => {
            const { argsPath } = request;
            return this.instance.delete(argsPath);
        };
        this.fireRequest = (request) => {
            const response = new ResponseBuilder(request);
            const apiMethod = this[request.argsMethod];
            if (typeof apiMethod !== 'function') {
                Promise.resolve(response);
            }
            return new Promise((resolve) => {
                apiMethod(request, response)
                    .then((res) => {
                    response.withIsSuccess(true).withAxiosResponse(res);
                    const data = this.parseReturnedData(request, res);
                    response.withData(data);
                    resolve(response.build());
                })
                    .catch(function (error) {
                    if (error.response) {
                        // The request was made and the server responded with a status code
                        // that falls out of the range of 2xx
                        const { status } = error.response;
                        const errorType = status === 401 ? 'authorization' : 'server';
                        response
                            .withErrorType(errorType)
                            .withAxiosResponse(error.response);
                    }
                    else if (error.request) {
                        // The request was made but no response was received
                        // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
                        // http.ClientRequest in node.js
                        response.withErrorType('timeout');
                    }
                    else {
                        // Something happened in setting up the request that triggered an Error
                        response
                            .withErrorType('javascript')
                            .withErrorMessage(error.message);
                    }
                    resolve(response.build());
                });
            });
        };
        this.instance = configuration.axios;
    }
    parseReturnedData(request, res) {
        const { data } = res;
        if (request.argsMethod === 'GET') {
            switch (request.argsNodeType) {
                case NodeType.COLLECTION_NODE:
                    return itemsToObject(data, request.argsParams?.page ?? 1, request.argsParams?.limit);
                default:
                    return data;
            }
        }
        else {
            return data;
        }
    }
}
/*
    checkConnection(url: string): Promise<boolean> {
        return new Promise((resolve) => {
            axios
                .get(url)
                .then(() => {
                    resolve(true);
                })
                .catch(function () {
                    resolve(false);
                });
        });
    }
    */
